/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi;

import io.typefox.lsapi.CompletionItemKind;
import io.typefox.lsapi.TextEdit;
import javax.annotation.Nullable;

/**
 * The Completion request is sent from the client to the server to compute completion items at a given cursor position.
 * Completion items are presented in the IntelliSense user interface. If computing complete completion items is expensive
 * servers can additional provide a handler for the resolve completion item request. This request is send when a
 * completion item is selected in the user interface.
 */
@SuppressWarnings("all")
public interface CompletionItem {
  /**
   * The label of this completion item. By default also the text that is inserted when selecting this completion.
   */
  public abstract String getLabel();
  
  /**
   * The kind of this completion item. Based of the kind an icon is chosen by the editor.
   */
  @Nullable
  public abstract CompletionItemKind getKind();
  
  /**
   * A human-readable string with additional information about this item, like type or symbol information.
   */
  @Nullable
  public abstract String getDetail();
  
  /**
   * A human-readable string that represents a doc-comment.
   */
  @Nullable
  public abstract String getDocumentation();
  
  /**
   * A string that shoud be used when comparing this item with other items. When `falsy` the label is used.
   */
  @Nullable
  public abstract String getSortText();
  
  /**
   * A string that should be used when filtering a set of completion items. When `falsy` the label is used.
   */
  @Nullable
  public abstract String getFilterText();
  
  /**
   * A string that should be inserted a document when selecting this completion. When `falsy` the label is used.
   */
  @Nullable
  public abstract String getInsertText();
  
  /**
   * An edit which is applied to a document when selecting this completion. When an edit is provided the value of
   * insertText is ignored.
   */
  @Nullable
  public abstract TextEdit getTextEdit();
  
  /**
   * An data entry field that is preserved on a completion item between a completion and a completion resolve request.
   */
  @Nullable
  public abstract Object getData();
}
