/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi;

import io.typefox.lsapi.CodeLensOptions;
import io.typefox.lsapi.CompletionOptions;
import io.typefox.lsapi.DocumentOnTypeFormattingOptions;
import io.typefox.lsapi.SignatureHelpOptions;
import io.typefox.lsapi.TextDocumentSyncKind;
import javax.annotation.Nullable;

@SuppressWarnings("all")
public interface ServerCapabilities {
  /**
   * Defines how text documents are synced.
   */
  @Nullable
  public abstract TextDocumentSyncKind getTextDocumentSync();
  
  /**
   * The server provides hover support.
   */
  @Nullable
  public abstract Boolean isHoverProvider();
  
  /**
   * The server provides completion support.
   */
  @Nullable
  public abstract CompletionOptions getCompletionProvider();
  
  /**
   * The server provides signature help support.
   */
  @Nullable
  public abstract SignatureHelpOptions getSignatureHelpProvider();
  
  /**
   * The server provides goto definition support.
   */
  @Nullable
  public abstract Boolean isDefinitionProvider();
  
  /**
   * The server provides find references support.
   */
  @Nullable
  public abstract Boolean isReferencesProvider();
  
  /**
   * The server provides document highlight support.
   */
  @Nullable
  public abstract Boolean isDocumentHighlightProvider();
  
  /**
   * The server provides document symbol support.
   */
  @Nullable
  public abstract Boolean isDocumentSymbolProvider();
  
  /**
   * The server provides workspace symbol support.
   */
  @Nullable
  public abstract Boolean isWorkspaceSymbolProvider();
  
  /**
   * The server provides code actions.
   */
  @Nullable
  public abstract Boolean isCodeActionProvider();
  
  /**
   * The server provides code lens.
   */
  @Nullable
  public abstract CodeLensOptions getCodeLensProvider();
  
  /**
   * The server provides document formatting.
   */
  @Nullable
  public abstract Boolean isDocumentFormattingProvider();
  
  /**
   * The server provides document range formatting.
   */
  @Nullable
  public abstract Boolean isDocumentRangeFormattingProvider();
  
  /**
   * The server provides document formatting on typing.
   */
  @Nullable
  public abstract DocumentOnTypeFormattingOptions getDocumentOnTypeFormattingProvider();
  
  /**
   * The server provides rename support.
   */
  @Nullable
  public abstract Boolean isRenameProvider();
}
