/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import com.google.common.collect.Lists;
import io.typefox.lsapi.CodeActionContext;
import io.typefox.lsapi.Diagnostic;
import io.typefox.lsapi.builders.DiagnosticBuilder;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.impl.CodeActionContextImpl;
import io.typefox.lsapi.impl.DiagnosticImpl;
import java.util.List;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link CodeActionContext}.
 */
@SuppressWarnings("all")
public class CodeActionContextBuilder implements IBuilder<CodeActionContext> {
  public CodeActionContextBuilder() {
    
  }
  
  public CodeActionContextBuilder(final Procedure1<CodeActionContextBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * An array of diagnostics.
   */
  private List<DiagnosticImpl> diagnostics = Lists.newArrayList();
  
  public CodeActionContextBuilder diagnostic(final Diagnostic diagnostic) {
    if (diagnostic != null && !(diagnostic instanceof DiagnosticImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + diagnostic.getClass().getSimpleName());
    this.diagnostics.add((DiagnosticImpl) diagnostic);
    return this;
  }
  
  public CodeActionContextBuilder diagnostic(final Procedure1<DiagnosticBuilder> initializer) {
    DiagnosticBuilder builder = new DiagnosticBuilder();
    initializer.apply(builder);
    return diagnostic(builder.build());
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public CodeActionContext build() {
    CodeActionContextImpl result = new CodeActionContextImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final CodeActionContextImpl result) {
    result.setDiagnostics(this.diagnostics);
  }
}
