/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.CodeLensOptions;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.impl.CodeLensOptionsImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link CodeLensOptions}.
 */
@SuppressWarnings("all")
public class CodeLensOptionsBuilder implements IBuilder<CodeLensOptions> {
  public CodeLensOptionsBuilder() {
    
  }
  
  public CodeLensOptionsBuilder(final Procedure1<CodeLensOptionsBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * Code lens has a resolve provider as well.
   */
  private boolean resolveProvider;
  
  public CodeLensOptionsBuilder resolveProvider(final boolean resolveProvider) {
    this.resolveProvider = resolveProvider;
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public CodeLensOptions build() {
    CodeLensOptionsImpl result = new CodeLensOptionsImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final CodeLensOptionsImpl result) {
    result.setResolveProvider(this.resolveProvider);
  }
}
