/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import com.google.common.collect.Lists;
import io.typefox.lsapi.Command;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.impl.CommandImpl;
import java.util.List;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link Command}.
 */
@SuppressWarnings("all")
public class CommandBuilder implements IBuilder<Command> {
  public CommandBuilder() {
    
  }
  
  public CommandBuilder(final Procedure1<CommandBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * Title of the command, like `save`.
   */
  private String title;
  
  public CommandBuilder title(final String title) {
    this.title = title;
    return this;
  }
  
  /**
   * The identifier of the actual command handler.
   */
  private String command;
  
  public CommandBuilder command(final String command) {
    this.command = command;
    return this;
  }
  
  /**
   * Arguments that the command handler should be invoked with.
   */
  private List<Object> arguments = Lists.newArrayList();
  
  public CommandBuilder argument(final Object argument) {
    this.arguments.add(argument);
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public Command build() {
    CommandImpl result = new CommandImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final CommandImpl result) {
    result.setTitle(this.title);
    result.setCommand(this.command);
    result.setArguments(this.arguments);
  }
}
