/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import com.google.common.collect.Lists;
import io.typefox.lsapi.CompletionItem;
import io.typefox.lsapi.CompletionList;
import io.typefox.lsapi.builders.CompletionItemBuilder;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.impl.CompletionItemImpl;
import io.typefox.lsapi.impl.CompletionListImpl;
import java.util.List;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link CompletionList}.
 */
@SuppressWarnings("all")
public class CompletionListBuilder implements IBuilder<CompletionList> {
  public CompletionListBuilder() {
    
  }
  
  public CompletionListBuilder(final Procedure1<CompletionListBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * This list it not complete. Further typing should result in recomputing this list.
   */
  private boolean incomplete;
  
  public CompletionListBuilder incomplete(final boolean incomplete) {
    this.incomplete = incomplete;
    return this;
  }
  
  /**
   * The completion items.
   */
  private List<CompletionItemImpl> items = Lists.newArrayList();
  
  public CompletionListBuilder item(final CompletionItem item) {
    if (item != null && !(item instanceof CompletionItemImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + item.getClass().getSimpleName());
    this.items.add((CompletionItemImpl) item);
    return this;
  }
  
  public CompletionListBuilder item(final Procedure1<CompletionItemBuilder> initializer) {
    CompletionItemBuilder builder = new CompletionItemBuilder();
    initializer.apply(builder);
    return item(builder.build());
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public CompletionList build() {
    CompletionListImpl result = new CompletionListImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final CompletionListImpl result) {
    result.setIncomplete(this.incomplete);
    result.setItems(this.items);
  }
}
