/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import com.google.common.collect.Lists;
import io.typefox.lsapi.CompletionOptions;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.impl.CompletionOptionsImpl;
import java.util.List;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link CompletionOptions}.
 */
@SuppressWarnings("all")
public class CompletionOptionsBuilder implements IBuilder<CompletionOptions> {
  public CompletionOptionsBuilder() {
    
  }
  
  public CompletionOptionsBuilder(final Procedure1<CompletionOptionsBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The server provides support to resolve additional information for a completion item.
   */
  private Boolean resolveProvider;
  
  public CompletionOptionsBuilder resolveProvider(final Boolean resolveProvider) {
    this.resolveProvider = resolveProvider;
    return this;
  }
  
  /**
   * The characters that trigger completion automatically.
   */
  private List<String> triggerCharacters = Lists.newArrayList();
  
  public CompletionOptionsBuilder triggerCharacter(final String triggerCharacter) {
    this.triggerCharacters.add(triggerCharacter);
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public CompletionOptions build() {
    CompletionOptionsImpl result = new CompletionOptionsImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final CompletionOptionsImpl result) {
    result.setResolveProvider(this.resolveProvider);
    result.setTriggerCharacters(this.triggerCharacters);
  }
}
