/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import com.google.common.collect.Lists;
import io.typefox.lsapi.DidChangeWatchedFilesParams;
import io.typefox.lsapi.FileChangeType;
import io.typefox.lsapi.FileEvent;
import io.typefox.lsapi.builders.FileEventBuilder;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.impl.DidChangeWatchedFilesParamsImpl;
import io.typefox.lsapi.impl.FileEventImpl;
import java.util.List;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link DidChangeWatchedFilesParams}.
 */
@SuppressWarnings("all")
public class DidChangeWatchedFilesParamsBuilder implements IBuilder<DidChangeWatchedFilesParams> {
  public DidChangeWatchedFilesParamsBuilder() {
    
  }
  
  public DidChangeWatchedFilesParamsBuilder(final Procedure1<DidChangeWatchedFilesParamsBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The actual file events.
   */
  private List<FileEventImpl> changes = Lists.newArrayList();
  
  public DidChangeWatchedFilesParamsBuilder change(final FileEvent change) {
    if (change != null && !(change instanceof FileEventImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + change.getClass().getSimpleName());
    this.changes.add((FileEventImpl) change);
    return this;
  }
  
  public DidChangeWatchedFilesParamsBuilder change(final Procedure1<FileEventBuilder> initializer) {
    FileEventBuilder builder = new FileEventBuilder();
    initializer.apply(builder);
    return change(builder.build());
  }
  
  public DidChangeWatchedFilesParamsBuilder change(final String uri, final FileChangeType type) {
    FileEvent instance = new FileEventImpl(uri, type);
    return change(instance);
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public DidChangeWatchedFilesParams build() {
    DidChangeWatchedFilesParamsImpl result = new DidChangeWatchedFilesParamsImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final DidChangeWatchedFilesParamsImpl result) {
    result.setChanges(this.changes);
  }
}
