/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.DidOpenTextDocumentParams;
import io.typefox.lsapi.TextDocumentItem;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.builders.TextDocumentItemBuilder;
import io.typefox.lsapi.impl.DidOpenTextDocumentParamsImpl;
import io.typefox.lsapi.impl.TextDocumentItemImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link DidOpenTextDocumentParams}.
 */
@SuppressWarnings("all")
public class DidOpenTextDocumentParamsBuilder implements IBuilder<DidOpenTextDocumentParams> {
  public DidOpenTextDocumentParamsBuilder() {
    
  }
  
  public DidOpenTextDocumentParamsBuilder(final Procedure1<DidOpenTextDocumentParamsBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The document that was opened.
   */
  private TextDocumentItemImpl textDocument;
  
  public DidOpenTextDocumentParamsBuilder textDocument(final TextDocumentItem textDocument) {
    if (textDocument != null && !(textDocument instanceof TextDocumentItemImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + textDocument.getClass().getSimpleName());
    this.textDocument = (TextDocumentItemImpl) textDocument;
    return this;
  }
  
  public DidOpenTextDocumentParamsBuilder textDocument(final Procedure1<TextDocumentItemBuilder> initializer) {
    TextDocumentItemBuilder builder = new TextDocumentItemBuilder();
    initializer.apply(builder);
    return textDocument(builder.build());
  }
  
  /**
   * Legacy property to support protocol version 1.0 requests.
   */
  private String text;
  
  public DidOpenTextDocumentParamsBuilder text(final String text) {
    this.text = text;
    return this;
  }
  
  /**
   * The text document's uri.
   */
  private String uri;
  
  public DidOpenTextDocumentParamsBuilder uri(final String uri) {
    this.uri = uri;
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public DidOpenTextDocumentParams build() {
    DidOpenTextDocumentParamsImpl result = new DidOpenTextDocumentParamsImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final DidOpenTextDocumentParamsImpl result) {
    result.setTextDocument(this.textDocument);
    result.setText(this.text);
    result.setUri(this.uri);
  }
}
