/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.DocumentFormattingParams;
import io.typefox.lsapi.FormattingOptions;
import io.typefox.lsapi.TextDocumentIdentifier;
import io.typefox.lsapi.builders.FormattingOptionsBuilder;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.builders.TextDocumentIdentifierBuilder;
import io.typefox.lsapi.impl.DocumentFormattingParamsImpl;
import io.typefox.lsapi.impl.FormattingOptionsImpl;
import io.typefox.lsapi.impl.TextDocumentIdentifierImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link DocumentFormattingParams}.
 */
@SuppressWarnings("all")
public class DocumentFormattingParamsBuilder implements IBuilder<DocumentFormattingParams> {
  public DocumentFormattingParamsBuilder() {
    
  }
  
  public DocumentFormattingParamsBuilder(final Procedure1<DocumentFormattingParamsBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The document to format.
   */
  private TextDocumentIdentifierImpl textDocument;
  
  public DocumentFormattingParamsBuilder textDocument(final TextDocumentIdentifier textDocument) {
    if (textDocument != null && !(textDocument instanceof TextDocumentIdentifierImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + textDocument.getClass().getSimpleName());
    this.textDocument = (TextDocumentIdentifierImpl) textDocument;
    return this;
  }
  
  public DocumentFormattingParamsBuilder textDocument(final Procedure1<TextDocumentIdentifierBuilder> initializer) {
    TextDocumentIdentifierBuilder builder = new TextDocumentIdentifierBuilder();
    initializer.apply(builder);
    return textDocument(builder.build());
  }
  
  public DocumentFormattingParamsBuilder textDocument(final String uri) {
    TextDocumentIdentifier instance = new TextDocumentIdentifierImpl(uri);
    return textDocument(instance);
  }
  
  /**
   * The format options
   */
  private FormattingOptionsImpl options;
  
  public DocumentFormattingParamsBuilder options(final FormattingOptions options) {
    if (options != null && !(options instanceof FormattingOptionsImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + options.getClass().getSimpleName());
    this.options = (FormattingOptionsImpl) options;
    return this;
  }
  
  public DocumentFormattingParamsBuilder options(final Procedure1<FormattingOptionsBuilder> initializer) {
    FormattingOptionsBuilder builder = new FormattingOptionsBuilder();
    initializer.apply(builder);
    return options(builder.build());
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public DocumentFormattingParams build() {
    DocumentFormattingParamsImpl result = new DocumentFormattingParamsImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final DocumentFormattingParamsImpl result) {
    result.setTextDocument(this.textDocument);
    result.setOptions(this.options);
  }
}
