/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.DocumentHighlight;
import io.typefox.lsapi.DocumentHighlightKind;
import io.typefox.lsapi.Position;
import io.typefox.lsapi.Range;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.builders.RangeBuilder;
import io.typefox.lsapi.impl.DocumentHighlightImpl;
import io.typefox.lsapi.impl.PositionImpl;
import io.typefox.lsapi.impl.RangeImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link DocumentHighlight}.
 */
@SuppressWarnings("all")
public class DocumentHighlightBuilder implements IBuilder<DocumentHighlight> {
  public DocumentHighlightBuilder() {
    
  }
  
  public DocumentHighlightBuilder(final Procedure1<DocumentHighlightBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The range this highlight applies to.
   */
  private RangeImpl range;
  
  public DocumentHighlightBuilder range(final Range range) {
    if (range != null && !(range instanceof RangeImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + range.getClass().getSimpleName());
    this.range = (RangeImpl) range;
    return this;
  }
  
  public DocumentHighlightBuilder range(final Procedure1<RangeBuilder> initializer) {
    RangeBuilder builder = new RangeBuilder();
    initializer.apply(builder);
    return range(builder.build());
  }
  
  public DocumentHighlightBuilder range(final Position start, final Position end) {
    if (start != null && !(start instanceof PositionImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + start.getClass().getSimpleName());
    if (end != null && !(end instanceof PositionImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + end.getClass().getSimpleName());
    Range instance = new RangeImpl((PositionImpl) start, (PositionImpl) end);
    return range(instance);
  }
  
  /**
   * The highlight kind, default is {@link DocumentHighlightKind#Text}.
   */
  private DocumentHighlightKind kind;
  
  public DocumentHighlightBuilder kind(final DocumentHighlightKind kind) {
    this.kind = kind;
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public DocumentHighlight build() {
    DocumentHighlightImpl result = new DocumentHighlightImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final DocumentHighlightImpl result) {
    result.setRange(this.range);
    result.setKind(this.kind);
  }
}
