/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import com.google.common.collect.Lists;
import io.typefox.lsapi.DocumentOnTypeFormattingOptions;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.impl.DocumentOnTypeFormattingOptionsImpl;
import java.util.List;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link DocumentOnTypeFormattingOptions}.
 */
@SuppressWarnings("all")
public class DocumentOnTypeFormattingOptionsBuilder implements IBuilder<DocumentOnTypeFormattingOptions> {
  public DocumentOnTypeFormattingOptionsBuilder() {
    
  }
  
  public DocumentOnTypeFormattingOptionsBuilder(final Procedure1<DocumentOnTypeFormattingOptionsBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * A character on which formatting should be triggered, like `}`.
   */
  private String firstTriggerCharacter;
  
  public DocumentOnTypeFormattingOptionsBuilder firstTriggerCharacter(final String firstTriggerCharacter) {
    this.firstTriggerCharacter = firstTriggerCharacter;
    return this;
  }
  
  /**
   * More trigger characters.
   */
  private List<String> moreTriggerCharacter = Lists.newArrayList();
  
  public DocumentOnTypeFormattingOptionsBuilder moreTriggerCharacter(final String moreTriggerCharacter) {
    this.moreTriggerCharacter.add(moreTriggerCharacter);
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public DocumentOnTypeFormattingOptions build() {
    DocumentOnTypeFormattingOptionsImpl result = new DocumentOnTypeFormattingOptionsImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final DocumentOnTypeFormattingOptionsImpl result) {
    result.setFirstTriggerCharacter(this.firstTriggerCharacter);
    result.setMoreTriggerCharacter(this.moreTriggerCharacter);
  }
}
