/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.DocumentSymbolParams;
import io.typefox.lsapi.TextDocumentIdentifier;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.builders.TextDocumentIdentifierBuilder;
import io.typefox.lsapi.impl.DocumentSymbolParamsImpl;
import io.typefox.lsapi.impl.TextDocumentIdentifierImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link DocumentSymbolParams}.
 */
@SuppressWarnings("all")
public class DocumentSymbolParamsBuilder implements IBuilder<DocumentSymbolParams> {
  public DocumentSymbolParamsBuilder() {
    
  }
  
  public DocumentSymbolParamsBuilder(final Procedure1<DocumentSymbolParamsBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The text document.
   */
  private TextDocumentIdentifierImpl textDocument;
  
  public DocumentSymbolParamsBuilder textDocument(final TextDocumentIdentifier textDocument) {
    if (textDocument != null && !(textDocument instanceof TextDocumentIdentifierImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + textDocument.getClass().getSimpleName());
    this.textDocument = (TextDocumentIdentifierImpl) textDocument;
    return this;
  }
  
  public DocumentSymbolParamsBuilder textDocument(final Procedure1<TextDocumentIdentifierBuilder> initializer) {
    TextDocumentIdentifierBuilder builder = new TextDocumentIdentifierBuilder();
    initializer.apply(builder);
    return textDocument(builder.build());
  }
  
  public DocumentSymbolParamsBuilder textDocument(final String uri) {
    TextDocumentIdentifier instance = new TextDocumentIdentifierImpl(uri);
    return textDocument(instance);
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public DocumentSymbolParams build() {
    DocumentSymbolParamsImpl result = new DocumentSymbolParamsImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final DocumentSymbolParamsImpl result) {
    result.setTextDocument(this.textDocument);
  }
}
