/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.FileChangeType;
import io.typefox.lsapi.FileEvent;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.impl.FileEventImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link FileEvent}.
 */
@SuppressWarnings("all")
public class FileEventBuilder implements IBuilder<FileEvent> {
  public FileEventBuilder() {
    
  }
  
  public FileEventBuilder(final Procedure1<FileEventBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The file's uri.
   */
  private String uri;
  
  public FileEventBuilder uri(final String uri) {
    this.uri = uri;
    return this;
  }
  
  /**
   * The change type.
   */
  private FileChangeType type;
  
  public FileEventBuilder type(final FileChangeType type) {
    this.type = type;
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public FileEvent build() {
    FileEventImpl result = new FileEventImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final FileEventImpl result) {
    result.setUri(this.uri);
    result.setType(this.type);
  }
}
