/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import com.google.common.collect.Maps;
import io.typefox.lsapi.FormattingOptions;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.impl.FormattingOptionsImpl;
import java.util.Map;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link FormattingOptions}.
 */
@SuppressWarnings("all")
public class FormattingOptionsBuilder implements IBuilder<FormattingOptions> {
  public FormattingOptionsBuilder() {
    
  }
  
  public FormattingOptionsBuilder(final Procedure1<FormattingOptionsBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * Size of a tab in spaces.
   */
  private int tabSize;
  
  public FormattingOptionsBuilder tabSize(final int tabSize) {
    this.tabSize = tabSize;
    return this;
  }
  
  /**
   * Prefer spaces over tabs.
   */
  private boolean insertSpaces;
  
  public FormattingOptionsBuilder insertSpaces(final boolean insertSpaces) {
    this.insertSpaces = insertSpaces;
    return this;
  }
  
  /**
   * Signature for further properties.
   */
  private Map<String, String> properties = Maps.newLinkedHashMap();
  
  public FormattingOptionsBuilder property(final String key, final String property) {
    if (this.properties == null)
    	this.properties = Maps.newLinkedHashMap();
    this.properties.put(key, property);
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public FormattingOptions build() {
    FormattingOptionsImpl result = new FormattingOptionsImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final FormattingOptionsImpl result) {
    result.setTabSize(this.tabSize);
    result.setInsertSpaces(this.insertSpaces);
    result.setProperties(this.properties);
  }
}
