/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.InitializeError;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.impl.InitializeErrorImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link InitializeError}.
 */
@SuppressWarnings("all")
public class InitializeErrorBuilder implements IBuilder<InitializeError> {
  public InitializeErrorBuilder() {
    
  }
  
  public InitializeErrorBuilder(final Procedure1<InitializeErrorBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * Indicates whether the client should retry to send the initialize request after showing the message provided
   * in the ResponseError.
   */
  private boolean retry;
  
  public InitializeErrorBuilder retry(final boolean retry) {
    this.retry = retry;
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public InitializeError build() {
    InitializeErrorImpl result = new InitializeErrorImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final InitializeErrorImpl result) {
    result.setRetry(this.retry);
  }
}
