/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.ClientCapabilities;
import io.typefox.lsapi.InitializeParams;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.impl.ClientCapabilitiesImpl;
import io.typefox.lsapi.impl.InitializeParamsImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link InitializeParams}.
 */
@SuppressWarnings("all")
public class InitializeParamsBuilder implements IBuilder<InitializeParams> {
  public InitializeParamsBuilder() {
    
  }
  
  public InitializeParamsBuilder(final Procedure1<InitializeParamsBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The process Id of the parent process that started the server.
   */
  private Integer processId;
  
  public InitializeParamsBuilder processId(final Integer processId) {
    this.processId = processId;
    return this;
  }
  
  /**
   * The rootPath of the workspace. Is null if no folder is open.
   */
  private String rootPath;
  
  public InitializeParamsBuilder rootPath(final String rootPath) {
    this.rootPath = rootPath;
    return this;
  }
  
  /**
   * The capabilities provided by the client (editor)
   */
  private ClientCapabilitiesImpl capabilities;
  
  public InitializeParamsBuilder capabilities(final ClientCapabilities capabilities) {
    if (capabilities != null && !(capabilities instanceof ClientCapabilitiesImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + capabilities.getClass().getSimpleName());
    this.capabilities = (ClientCapabilitiesImpl) capabilities;
    return this;
  }
  
  /**
   * An optional extension to the protocol.
   * To tell the server what client (editor) is talking to it.
   */
  private String clientName;
  
  public InitializeParamsBuilder clientName(final String clientName) {
    this.clientName = clientName;
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public InitializeParams build() {
    InitializeParamsImpl result = new InitializeParamsImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final InitializeParamsImpl result) {
    result.setProcessId(this.processId);
    result.setRootPath(this.rootPath);
    result.setCapabilities(this.capabilities);
    result.setClientName(this.clientName);
  }
}
