/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.InitializeResult;
import io.typefox.lsapi.ServerCapabilities;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.builders.ServerCapabilitiesBuilder;
import io.typefox.lsapi.impl.InitializeResultImpl;
import io.typefox.lsapi.impl.ServerCapabilitiesImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link InitializeResult}.
 */
@SuppressWarnings("all")
public class InitializeResultBuilder implements IBuilder<InitializeResult> {
  public InitializeResultBuilder() {
    
  }
  
  public InitializeResultBuilder(final Procedure1<InitializeResultBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The capabilities the language server provides.
   */
  private ServerCapabilitiesImpl capabilities;
  
  public InitializeResultBuilder capabilities(final ServerCapabilities capabilities) {
    if (capabilities != null && !(capabilities instanceof ServerCapabilitiesImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + capabilities.getClass().getSimpleName());
    this.capabilities = (ServerCapabilitiesImpl) capabilities;
    return this;
  }
  
  public InitializeResultBuilder capabilities(final Procedure1<ServerCapabilitiesBuilder> initializer) {
    ServerCapabilitiesBuilder builder = new ServerCapabilitiesBuilder();
    initializer.apply(builder);
    return capabilities(builder.build());
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public InitializeResult build() {
    InitializeResultImpl result = new InitializeResultImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final InitializeResultImpl result) {
    result.setCapabilities(this.capabilities);
  }
}
