/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.Location;
import io.typefox.lsapi.Position;
import io.typefox.lsapi.Range;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.builders.RangeBuilder;
import io.typefox.lsapi.impl.LocationImpl;
import io.typefox.lsapi.impl.PositionImpl;
import io.typefox.lsapi.impl.RangeImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link Location}.
 */
@SuppressWarnings("all")
public class LocationBuilder implements IBuilder<Location> {
  public LocationBuilder() {
    
  }
  
  public LocationBuilder(final Procedure1<LocationBuilder> initializer) {
    initializer.apply(this);
  }
  
  private String uri;
  
  public LocationBuilder uri(final String uri) {
    this.uri = uri;
    return this;
  }
  
  private RangeImpl range;
  
  public LocationBuilder range(final Range range) {
    if (range != null && !(range instanceof RangeImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + range.getClass().getSimpleName());
    this.range = (RangeImpl) range;
    return this;
  }
  
  public LocationBuilder range(final Procedure1<RangeBuilder> initializer) {
    RangeBuilder builder = new RangeBuilder();
    initializer.apply(builder);
    return range(builder.build());
  }
  
  public LocationBuilder range(final Position start, final Position end) {
    if (start != null && !(start instanceof PositionImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + start.getClass().getSimpleName());
    if (end != null && !(end instanceof PositionImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + end.getClass().getSimpleName());
    Range instance = new RangeImpl((PositionImpl) start, (PositionImpl) end);
    return range(instance);
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public Location build() {
    LocationImpl result = new LocationImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final LocationImpl result) {
    result.setUri(this.uri);
    result.setRange(this.range);
  }
}
