/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.MarkedString;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.impl.MarkedStringImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link MarkedString}.
 */
@SuppressWarnings("all")
public class MarkedStringBuilder implements IBuilder<MarkedString> {
  public MarkedStringBuilder() {
    
  }
  
  public MarkedStringBuilder(final Procedure1<MarkedStringBuilder> initializer) {
    initializer.apply(this);
  }
  
  private String language;
  
  public MarkedStringBuilder language(final String language) {
    this.language = language;
    return this;
  }
  
  private String value;
  
  public MarkedStringBuilder value(final String value) {
    this.value = value;
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public MarkedString build() {
    MarkedStringImpl result = new MarkedStringImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final MarkedStringImpl result) {
    result.setLanguage(this.language);
    result.setValue(this.value);
  }
}
