/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.MessageActionItem;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.impl.MessageActionItemImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link MessageActionItem}.
 */
@SuppressWarnings("all")
public class MessageActionItemBuilder implements IBuilder<MessageActionItem> {
  public MessageActionItemBuilder() {
    
  }
  
  public MessageActionItemBuilder(final Procedure1<MessageActionItemBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * A short title like 'Retry', 'Open Log' etc.
   */
  private String title;
  
  public MessageActionItemBuilder title(final String title) {
    this.title = title;
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public MessageActionItem build() {
    MessageActionItemImpl result = new MessageActionItemImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final MessageActionItemImpl result) {
    result.setTitle(this.title);
  }
}
