/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.Message;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.impl.MessageImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link Message}.
 */
@SuppressWarnings("all")
public class MessageBuilder implements IBuilder<Message> {
  public MessageBuilder() {
    
  }
  
  public MessageBuilder(final Procedure1<MessageBuilder> initializer) {
    initializer.apply(this);
  }
  
  private String jsonrpc;
  
  public MessageBuilder jsonrpc(final String jsonrpc) {
    this.jsonrpc = jsonrpc;
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public Message build() {
    MessageImpl result = new MessageImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final MessageImpl result) {
    result.setJsonrpc(this.jsonrpc);
  }
}
