/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.MessageParams;
import io.typefox.lsapi.MessageType;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.impl.MessageParamsImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link MessageParams}.
 */
@SuppressWarnings("all")
public class MessageParamsBuilder implements IBuilder<MessageParams> {
  public MessageParamsBuilder() {
    
  }
  
  public MessageParamsBuilder(final Procedure1<MessageParamsBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The message type.
   */
  private MessageType type;
  
  public MessageParamsBuilder type(final MessageType type) {
    this.type = type;
    return this;
  }
  
  /**
   * The actual message.
   */
  private String message;
  
  public MessageParamsBuilder message(final String message) {
    this.message = message;
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public MessageParams build() {
    MessageParamsImpl result = new MessageParamsImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final MessageParamsImpl result) {
    result.setType(this.type);
    result.setMessage(this.message);
  }
}
