/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.NotificationMessage;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.impl.NotificationMessageImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link NotificationMessage}.
 */
@SuppressWarnings("all")
public class NotificationMessageBuilder implements IBuilder<NotificationMessage> {
  public NotificationMessageBuilder() {
    
  }
  
  public NotificationMessageBuilder(final Procedure1<NotificationMessageBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The method to be invoked.
   */
  private String method;
  
  public NotificationMessageBuilder method(final String method) {
    this.method = method;
    return this;
  }
  
  /**
   * The notification's params.
   */
  private Object params;
  
  public NotificationMessageBuilder params(final Object params) {
    this.params = params;
    return this;
  }
  
  private String jsonrpc;
  
  public NotificationMessageBuilder jsonrpc(final String jsonrpc) {
    this.jsonrpc = jsonrpc;
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public NotificationMessage build() {
    NotificationMessageImpl result = new NotificationMessageImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final NotificationMessageImpl result) {
    result.setMethod(this.method);
    result.setParams(this.params);
    result.setJsonrpc(this.jsonrpc);
  }
}
