/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.ParameterInformation;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.impl.ParameterInformationImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link ParameterInformation}.
 */
@SuppressWarnings("all")
public class ParameterInformationBuilder implements IBuilder<ParameterInformation> {
  public ParameterInformationBuilder() {
    
  }
  
  public ParameterInformationBuilder(final Procedure1<ParameterInformationBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The label of this signature. Will be shown in the UI.
   */
  private String label;
  
  public ParameterInformationBuilder label(final String label) {
    this.label = label;
    return this;
  }
  
  /**
   * The human-readable doc-comment of this signature. Will be shown in the UI but can be omitted.
   */
  private String documentation;
  
  public ParameterInformationBuilder documentation(final String documentation) {
    this.documentation = documentation;
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public ParameterInformation build() {
    ParameterInformationImpl result = new ParameterInformationImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final ParameterInformationImpl result) {
    result.setLabel(this.label);
    result.setDocumentation(this.documentation);
  }
}
