/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.Position;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.impl.PositionImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link Position}.
 */
@SuppressWarnings("all")
public class PositionBuilder implements IBuilder<Position> {
  public PositionBuilder() {
    
  }
  
  public PositionBuilder(final Procedure1<PositionBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * Line position in a document (zero-based).
   */
  private int line;
  
  public PositionBuilder line(final int line) {
    this.line = line;
    return this;
  }
  
  /**
   * Character offset on a line in a document (zero-based).
   */
  private int character;
  
  public PositionBuilder character(final int character) {
    this.character = character;
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public Position build() {
    PositionImpl result = new PositionImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final PositionImpl result) {
    result.setLine(this.line);
    result.setCharacter(this.character);
  }
}
