/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import com.google.common.collect.Lists;
import io.typefox.lsapi.Diagnostic;
import io.typefox.lsapi.PublishDiagnosticsParams;
import io.typefox.lsapi.builders.DiagnosticBuilder;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.impl.DiagnosticImpl;
import io.typefox.lsapi.impl.PublishDiagnosticsParamsImpl;
import java.util.List;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link PublishDiagnosticsParams}.
 */
@SuppressWarnings("all")
public class PublishDiagnosticsParamsBuilder implements IBuilder<PublishDiagnosticsParams> {
  public PublishDiagnosticsParamsBuilder() {
    
  }
  
  public PublishDiagnosticsParamsBuilder(final Procedure1<PublishDiagnosticsParamsBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The URI for which diagnostic information is reported.
   */
  private String uri;
  
  public PublishDiagnosticsParamsBuilder uri(final String uri) {
    this.uri = uri;
    return this;
  }
  
  /**
   * An array of diagnostic information items.
   */
  private List<DiagnosticImpl> diagnostics = Lists.newArrayList();
  
  public PublishDiagnosticsParamsBuilder diagnostic(final Diagnostic diagnostic) {
    if (diagnostic != null && !(diagnostic instanceof DiagnosticImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + diagnostic.getClass().getSimpleName());
    this.diagnostics.add((DiagnosticImpl) diagnostic);
    return this;
  }
  
  public PublishDiagnosticsParamsBuilder diagnostic(final Procedure1<DiagnosticBuilder> initializer) {
    DiagnosticBuilder builder = new DiagnosticBuilder();
    initializer.apply(builder);
    return diagnostic(builder.build());
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public PublishDiagnosticsParams build() {
    PublishDiagnosticsParamsImpl result = new PublishDiagnosticsParamsImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final PublishDiagnosticsParamsImpl result) {
    result.setUri(this.uri);
    result.setDiagnostics(this.diagnostics);
  }
}
