/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.Position;
import io.typefox.lsapi.Range;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.builders.PositionBuilder;
import io.typefox.lsapi.impl.PositionImpl;
import io.typefox.lsapi.impl.RangeImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link Range}.
 */
@SuppressWarnings("all")
public class RangeBuilder implements IBuilder<Range> {
  public RangeBuilder() {
    
  }
  
  public RangeBuilder(final Procedure1<RangeBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The range's start position
   */
  private PositionImpl start;
  
  public RangeBuilder start(final Position start) {
    if (start != null && !(start instanceof PositionImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + start.getClass().getSimpleName());
    this.start = (PositionImpl) start;
    return this;
  }
  
  public RangeBuilder start(final Procedure1<PositionBuilder> initializer) {
    PositionBuilder builder = new PositionBuilder();
    initializer.apply(builder);
    return start(builder.build());
  }
  
  public RangeBuilder start(final int line, final int character) {
    Position instance = new PositionImpl(line, character);
    return start(instance);
  }
  
  /**
   * The range's end position
   */
  private PositionImpl end;
  
  public RangeBuilder end(final Position end) {
    if (end != null && !(end instanceof PositionImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + end.getClass().getSimpleName());
    this.end = (PositionImpl) end;
    return this;
  }
  
  public RangeBuilder end(final Procedure1<PositionBuilder> initializer) {
    PositionBuilder builder = new PositionBuilder();
    initializer.apply(builder);
    return end(builder.build());
  }
  
  public RangeBuilder end(final int line, final int character) {
    Position instance = new PositionImpl(line, character);
    return end(instance);
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public Range build() {
    RangeImpl result = new RangeImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final RangeImpl result) {
    result.setStart(this.start);
    result.setEnd(this.end);
  }
}
