/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.ReferenceContext;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.impl.ReferenceContextImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link ReferenceContext}.
 */
@SuppressWarnings("all")
public class ReferenceContextBuilder implements IBuilder<ReferenceContext> {
  public ReferenceContextBuilder() {
    
  }
  
  public ReferenceContextBuilder(final Procedure1<ReferenceContextBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * Include the declaration of the current symbol.
   */
  private boolean includeDeclaration;
  
  public ReferenceContextBuilder includeDeclaration(final boolean includeDeclaration) {
    this.includeDeclaration = includeDeclaration;
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public ReferenceContext build() {
    ReferenceContextImpl result = new ReferenceContextImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final ReferenceContextImpl result) {
    result.setIncludeDeclaration(this.includeDeclaration);
  }
}
