/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.RequestMessage;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.impl.RequestMessageImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link RequestMessage}.
 */
@SuppressWarnings("all")
public class RequestMessageBuilder implements IBuilder<RequestMessage> {
  public RequestMessageBuilder() {
    
  }
  
  public RequestMessageBuilder(final Procedure1<RequestMessageBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The request id.
   */
  private String id;
  
  public RequestMessageBuilder id(final String id) {
    this.id = id;
    return this;
  }
  
  /**
   * The method to be invoked.
   */
  private String method;
  
  public RequestMessageBuilder method(final String method) {
    this.method = method;
    return this;
  }
  
  /**
   * The method's params.
   */
  private Object params;
  
  public RequestMessageBuilder params(final Object params) {
    this.params = params;
    return this;
  }
  
  private String jsonrpc;
  
  public RequestMessageBuilder jsonrpc(final String jsonrpc) {
    this.jsonrpc = jsonrpc;
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public RequestMessage build() {
    RequestMessageImpl result = new RequestMessageImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final RequestMessageImpl result) {
    result.setId(this.id);
    result.setMethod(this.method);
    result.setParams(this.params);
    result.setJsonrpc(this.jsonrpc);
  }
}
