/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.ResponseError;
import io.typefox.lsapi.ResponseErrorCode;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.impl.ResponseErrorImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link ResponseError}.
 */
@SuppressWarnings("all")
public class ResponseErrorBuilder implements IBuilder<ResponseError> {
  public ResponseErrorBuilder() {
    
  }
  
  public ResponseErrorBuilder(final Procedure1<ResponseErrorBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * A number indicating the error type that occured.
   */
  private ResponseErrorCode code;
  
  public ResponseErrorBuilder code(final ResponseErrorCode code) {
    this.code = code;
    return this;
  }
  
  /**
   * A string providing a short decription of the error.
   */
  private String message;
  
  public ResponseErrorBuilder message(final String message) {
    this.message = message;
    return this;
  }
  
  /**
   * A Primitive or Structured value that contains additional information about the error. Can be omitted.
   */
  private Object data;
  
  public ResponseErrorBuilder data(final Object data) {
    this.data = data;
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public ResponseError build() {
    ResponseErrorImpl result = new ResponseErrorImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final ResponseErrorImpl result) {
    result.setCode(this.code);
    result.setMessage(this.message);
    result.setData(this.data);
  }
}
