/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.ResponseError;
import io.typefox.lsapi.ResponseErrorCode;
import io.typefox.lsapi.ResponseMessage;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.builders.ResponseErrorBuilder;
import io.typefox.lsapi.impl.ResponseErrorImpl;
import io.typefox.lsapi.impl.ResponseMessageImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link ResponseMessage}.
 */
@SuppressWarnings("all")
public class ResponseMessageBuilder implements IBuilder<ResponseMessage> {
  public ResponseMessageBuilder() {
    
  }
  
  public ResponseMessageBuilder(final Procedure1<ResponseMessageBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The request id.
   */
  private String id;
  
  public ResponseMessageBuilder id(final String id) {
    this.id = id;
    return this;
  }
  
  /**
   * The result of a request. This can be omitted in the case of an error.
   */
  private Object result;
  
  public ResponseMessageBuilder result(final Object result) {
    this.result = result;
    return this;
  }
  
  /**
   * The error object in case a request fails.
   */
  private ResponseErrorImpl error;
  
  public ResponseMessageBuilder error(final ResponseError error) {
    if (error != null && !(error instanceof ResponseErrorImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + error.getClass().getSimpleName());
    this.error = (ResponseErrorImpl) error;
    return this;
  }
  
  public ResponseMessageBuilder error(final Procedure1<ResponseErrorBuilder> initializer) {
    ResponseErrorBuilder builder = new ResponseErrorBuilder();
    initializer.apply(builder);
    return error(builder.build());
  }
  
  public ResponseMessageBuilder error(final ResponseErrorCode code, final String message, final Object data) {
    ResponseError instance = new ResponseErrorImpl(code, message, data);
    return error(instance);
  }
  
  private String jsonrpc;
  
  public ResponseMessageBuilder jsonrpc(final String jsonrpc) {
    this.jsonrpc = jsonrpc;
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public ResponseMessage build() {
    ResponseMessageImpl result = new ResponseMessageImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final ResponseMessageImpl result) {
    result.setId(this.id);
    result.setResult(this.result);
    result.setError(this.error);
    result.setJsonrpc(this.jsonrpc);
  }
}
