/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import com.google.common.collect.Lists;
import io.typefox.lsapi.MessageActionItem;
import io.typefox.lsapi.MessageType;
import io.typefox.lsapi.ShowMessageRequestParams;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.builders.MessageActionItemBuilder;
import io.typefox.lsapi.impl.MessageActionItemImpl;
import io.typefox.lsapi.impl.ShowMessageRequestParamsImpl;
import java.util.List;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link ShowMessageRequestParams}.
 */
@SuppressWarnings("all")
public class ShowMessageRequestParamsBuilder implements IBuilder<ShowMessageRequestParams> {
  public ShowMessageRequestParamsBuilder() {
    
  }
  
  public ShowMessageRequestParamsBuilder(final Procedure1<ShowMessageRequestParamsBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The message action items to present.
   */
  private List<MessageActionItemImpl> actions = Lists.newArrayList();
  
  public ShowMessageRequestParamsBuilder action(final MessageActionItem action) {
    if (action != null && !(action instanceof MessageActionItemImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + action.getClass().getSimpleName());
    this.actions.add((MessageActionItemImpl) action);
    return this;
  }
  
  public ShowMessageRequestParamsBuilder action(final Procedure1<MessageActionItemBuilder> initializer) {
    MessageActionItemBuilder builder = new MessageActionItemBuilder();
    initializer.apply(builder);
    return action(builder.build());
  }
  
  public ShowMessageRequestParamsBuilder action(final String title) {
    MessageActionItem instance = new MessageActionItemImpl(title);
    return action(instance);
  }
  
  /**
   * The message type.
   */
  private MessageType type;
  
  public ShowMessageRequestParamsBuilder type(final MessageType type) {
    this.type = type;
    return this;
  }
  
  /**
   * The actual message.
   */
  private String message;
  
  public ShowMessageRequestParamsBuilder message(final String message) {
    this.message = message;
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public ShowMessageRequestParams build() {
    ShowMessageRequestParamsImpl result = new ShowMessageRequestParamsImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final ShowMessageRequestParamsImpl result) {
    result.setActions(this.actions);
    result.setType(this.type);
    result.setMessage(this.message);
  }
}
