/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import com.google.common.collect.Lists;
import io.typefox.lsapi.SignatureHelp;
import io.typefox.lsapi.SignatureInformation;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.builders.SignatureInformationBuilder;
import io.typefox.lsapi.impl.SignatureHelpImpl;
import io.typefox.lsapi.impl.SignatureInformationImpl;
import java.util.List;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link SignatureHelp}.
 */
@SuppressWarnings("all")
public class SignatureHelpBuilder implements IBuilder<SignatureHelp> {
  public SignatureHelpBuilder() {
    
  }
  
  public SignatureHelpBuilder(final Procedure1<SignatureHelpBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * One or more signatures.
   */
  private List<SignatureInformationImpl> signatures = Lists.newArrayList();
  
  public SignatureHelpBuilder signature(final SignatureInformation signature) {
    if (signature != null && !(signature instanceof SignatureInformationImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + signature.getClass().getSimpleName());
    this.signatures.add((SignatureInformationImpl) signature);
    return this;
  }
  
  public SignatureHelpBuilder signature(final Procedure1<SignatureInformationBuilder> initializer) {
    SignatureInformationBuilder builder = new SignatureInformationBuilder();
    initializer.apply(builder);
    return signature(builder.build());
  }
  
  /**
   * The active signature.
   */
  private Integer activeSignature;
  
  public SignatureHelpBuilder activeSignature(final Integer activeSignature) {
    this.activeSignature = activeSignature;
    return this;
  }
  
  /**
   * The active parameter of the active signature.
   */
  private Integer activeParameter;
  
  public SignatureHelpBuilder activeParameter(final Integer activeParameter) {
    this.activeParameter = activeParameter;
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public SignatureHelp build() {
    SignatureHelpImpl result = new SignatureHelpImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final SignatureHelpImpl result) {
    result.setSignatures(this.signatures);
    result.setActiveSignature(this.activeSignature);
    result.setActiveParameter(this.activeParameter);
  }
}
