/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import com.google.common.collect.Lists;
import io.typefox.lsapi.SignatureHelpOptions;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.impl.SignatureHelpOptionsImpl;
import java.util.List;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link SignatureHelpOptions}.
 */
@SuppressWarnings("all")
public class SignatureHelpOptionsBuilder implements IBuilder<SignatureHelpOptions> {
  public SignatureHelpOptionsBuilder() {
    
  }
  
  public SignatureHelpOptionsBuilder(final Procedure1<SignatureHelpOptionsBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The characters that trigger signature help automatically.
   */
  private List<String> triggerCharacters = Lists.newArrayList();
  
  public SignatureHelpOptionsBuilder triggerCharacter(final String triggerCharacter) {
    this.triggerCharacters.add(triggerCharacter);
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public SignatureHelpOptions build() {
    SignatureHelpOptionsImpl result = new SignatureHelpOptionsImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final SignatureHelpOptionsImpl result) {
    result.setTriggerCharacters(this.triggerCharacters);
  }
}
