/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.Position;
import io.typefox.lsapi.Range;
import io.typefox.lsapi.TextDocumentContentChangeEvent;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.builders.RangeBuilder;
import io.typefox.lsapi.impl.PositionImpl;
import io.typefox.lsapi.impl.RangeImpl;
import io.typefox.lsapi.impl.TextDocumentContentChangeEventImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link TextDocumentContentChangeEvent}.
 */
@SuppressWarnings("all")
public class TextDocumentContentChangeEventBuilder implements IBuilder<TextDocumentContentChangeEvent> {
  public TextDocumentContentChangeEventBuilder() {
    
  }
  
  public TextDocumentContentChangeEventBuilder(final Procedure1<TextDocumentContentChangeEventBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The range of the document that changed.
   */
  private RangeImpl range;
  
  public TextDocumentContentChangeEventBuilder range(final Range range) {
    if (range != null && !(range instanceof RangeImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + range.getClass().getSimpleName());
    this.range = (RangeImpl) range;
    return this;
  }
  
  public TextDocumentContentChangeEventBuilder range(final Procedure1<RangeBuilder> initializer) {
    RangeBuilder builder = new RangeBuilder();
    initializer.apply(builder);
    return range(builder.build());
  }
  
  public TextDocumentContentChangeEventBuilder range(final Position start, final Position end) {
    if (start != null && !(start instanceof PositionImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + start.getClass().getSimpleName());
    if (end != null && !(end instanceof PositionImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + end.getClass().getSimpleName());
    Range instance = new RangeImpl((PositionImpl) start, (PositionImpl) end);
    return range(instance);
  }
  
  /**
   * The length of the range that got replaced.
   */
  private Integer rangeLength;
  
  public TextDocumentContentChangeEventBuilder rangeLength(final Integer rangeLength) {
    this.rangeLength = rangeLength;
    return this;
  }
  
  /**
   * The new text of the document.
   */
  private String text;
  
  public TextDocumentContentChangeEventBuilder text(final String text) {
    this.text = text;
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public TextDocumentContentChangeEvent build() {
    TextDocumentContentChangeEventImpl result = new TextDocumentContentChangeEventImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final TextDocumentContentChangeEventImpl result) {
    result.setRange(this.range);
    result.setRangeLength(this.rangeLength);
    result.setText(this.text);
  }
}
