/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.TextDocumentItem;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.impl.TextDocumentItemImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link TextDocumentItem}.
 */
@SuppressWarnings("all")
public class TextDocumentItemBuilder implements IBuilder<TextDocumentItem> {
  public TextDocumentItemBuilder() {
    
  }
  
  public TextDocumentItemBuilder(final Procedure1<TextDocumentItemBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The text document's uri.
   */
  private String uri;
  
  public TextDocumentItemBuilder uri(final String uri) {
    this.uri = uri;
    return this;
  }
  
  /**
   * The text document's language identifier
   */
  private String languageId;
  
  public TextDocumentItemBuilder languageId(final String languageId) {
    this.languageId = languageId;
    return this;
  }
  
  /**
   * The version number of this document (it will strictly increase after each change, including undo/redo).
   */
  private int version;
  
  public TextDocumentItemBuilder version(final int version) {
    this.version = version;
    return this;
  }
  
  /**
   * The content of the opened  text document.
   */
  private String text;
  
  public TextDocumentItemBuilder text(final String text) {
    this.text = text;
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public TextDocumentItem build() {
    TextDocumentItemImpl result = new TextDocumentItemImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final TextDocumentItemImpl result) {
    result.setUri(this.uri);
    result.setLanguageId(this.languageId);
    result.setVersion(this.version);
    result.setText(this.text);
  }
}
