/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.Position;
import io.typefox.lsapi.Range;
import io.typefox.lsapi.TextEdit;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.builders.RangeBuilder;
import io.typefox.lsapi.impl.PositionImpl;
import io.typefox.lsapi.impl.RangeImpl;
import io.typefox.lsapi.impl.TextEditImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link TextEdit}.
 */
@SuppressWarnings("all")
public class TextEditBuilder implements IBuilder<TextEdit> {
  public TextEditBuilder() {
    
  }
  
  public TextEditBuilder(final Procedure1<TextEditBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The range of the text document to be manipulated. To insert text into a document create a range where start === end.
   */
  private RangeImpl range;
  
  public TextEditBuilder range(final Range range) {
    if (range != null && !(range instanceof RangeImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + range.getClass().getSimpleName());
    this.range = (RangeImpl) range;
    return this;
  }
  
  public TextEditBuilder range(final Procedure1<RangeBuilder> initializer) {
    RangeBuilder builder = new RangeBuilder();
    initializer.apply(builder);
    return range(builder.build());
  }
  
  public TextEditBuilder range(final Position start, final Position end) {
    if (start != null && !(start instanceof PositionImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + start.getClass().getSimpleName());
    if (end != null && !(end instanceof PositionImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + end.getClass().getSimpleName());
    Range instance = new RangeImpl((PositionImpl) start, (PositionImpl) end);
    return range(instance);
  }
  
  /**
   * The string to be inserted. For delete operations use an empty string.
   */
  private String newText;
  
  public TextEditBuilder newText(final String newText) {
    this.newText = newText;
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public TextEdit build() {
    TextEditImpl result = new TextEditImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final TextEditImpl result) {
    result.setRange(this.range);
    result.setNewText(this.newText);
  }
}
