/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.VersionedTextDocumentIdentifier;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.impl.VersionedTextDocumentIdentifierImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link VersionedTextDocumentIdentifier}.
 */
@SuppressWarnings("all")
public class VersionedTextDocumentIdentifierBuilder implements IBuilder<VersionedTextDocumentIdentifier> {
  public VersionedTextDocumentIdentifierBuilder() {
    
  }
  
  public VersionedTextDocumentIdentifierBuilder(final Procedure1<VersionedTextDocumentIdentifierBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * The version number of this document.
   */
  private int version;
  
  public VersionedTextDocumentIdentifierBuilder version(final int version) {
    this.version = version;
    return this;
  }
  
  /**
   * The text document's uri.
   */
  private String uri;
  
  public VersionedTextDocumentIdentifierBuilder uri(final String uri) {
    this.uri = uri;
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public VersionedTextDocumentIdentifier build() {
    VersionedTextDocumentIdentifierImpl result = new VersionedTextDocumentIdentifierImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final VersionedTextDocumentIdentifierImpl result) {
    result.setVersion(this.version);
    result.setUri(this.uri);
  }
}
