/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import io.typefox.lsapi.Range;
import io.typefox.lsapi.TextEdit;
import io.typefox.lsapi.WorkspaceEdit;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.builders.TextEditBuilder;
import io.typefox.lsapi.impl.RangeImpl;
import io.typefox.lsapi.impl.TextEditImpl;
import io.typefox.lsapi.impl.WorkspaceEditImpl;
import java.util.List;
import java.util.Map;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link WorkspaceEdit}.
 */
@SuppressWarnings("all")
public class WorkspaceEditBuilder implements IBuilder<WorkspaceEdit> {
  public WorkspaceEditBuilder() {
    
  }
  
  public WorkspaceEditBuilder(final Procedure1<WorkspaceEditBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * Holds changes to existing resources.
   */
  private Map<String, List<TextEditImpl>> changes = Maps.newLinkedHashMap();
  
  public WorkspaceEditBuilder change(final String key, final TextEdit change) {
    List<TextEditImpl> list = this.changes.get(key);
    if (list == null) {
    	list = Lists.newArrayList();
    	this.changes.put(key, list);
    }
    if (change != null && !(change instanceof TextEditImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + change.getClass().getSimpleName());
    list.add((TextEditImpl) change);
    return this;
  }
  
  public WorkspaceEditBuilder change(final String key, final Procedure1<TextEditBuilder> initializer) {
    TextEditBuilder builder = new TextEditBuilder();
    initializer.apply(builder);
    return change(key, builder.build());
  }
  
  public WorkspaceEditBuilder change(final String key, final Range range, final String newText) {
    if (range != null && !(range instanceof RangeImpl))
    	throw new IllegalArgumentException("Implementation not supported: " + range.getClass().getSimpleName());
    TextEdit instance = new TextEditImpl((RangeImpl) range, newText);
    return change(key, instance);
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public WorkspaceEdit build() {
    WorkspaceEditImpl result = new WorkspaceEditImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final WorkspaceEditImpl result) {
    result.setChanges(this.changes);
  }
}
