/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.builders;

import io.typefox.lsapi.WorkspaceSymbolParams;
import io.typefox.lsapi.builders.IBuilder;
import io.typefox.lsapi.impl.WorkspaceSymbolParamsImpl;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Builder for instances of {@link WorkspaceSymbolParams}.
 */
@SuppressWarnings("all")
public class WorkspaceSymbolParamsBuilder implements IBuilder<WorkspaceSymbolParams> {
  public WorkspaceSymbolParamsBuilder() {
    
  }
  
  public WorkspaceSymbolParamsBuilder(final Procedure1<WorkspaceSymbolParamsBuilder> initializer) {
    initializer.apply(this);
  }
  
  /**
   * A non-empty query string
   */
  private String query;
  
  public WorkspaceSymbolParamsBuilder query(final String query) {
    this.query = query;
    return this;
  }
  
  /**
   * Build the configured instance.
   */
  @Override
  public WorkspaceSymbolParams build() {
    WorkspaceSymbolParamsImpl result = new WorkspaceSymbolParamsImpl();
    internalBuild(result);
    return result;
  }
  
  protected void internalBuild(final WorkspaceSymbolParamsImpl result) {
    result.setQuery(this.query);
  }
}
