/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.impl;

import io.typefox.lsapi.DidOpenTextDocumentParams;
import io.typefox.lsapi.impl.TextDocumentIdentifierImpl;
import io.typefox.lsapi.impl.TextDocumentItemImpl;
import javax.annotation.Nullable;
import org.eclipse.xtext.xbase.lib.Pure;
import org.eclipse.xtext.xbase.lib.util.ToStringBuilder;

/**
 * The document open notification is sent from the client to the server to signal newly opened text documents.
 * The document's truth is now managed by the client and the server must not try to read the document's truth using
 * the document's uri.
 */
@SuppressWarnings("all")
public class DidOpenTextDocumentParamsImpl extends TextDocumentIdentifierImpl implements DidOpenTextDocumentParams {
  /**
   * The document that was opened.
   */
  private TextDocumentItemImpl textDocument;
  
  @Pure
  @Override
  public TextDocumentItemImpl getTextDocument() {
    return this.textDocument;
  }
  
  public void setTextDocument(final TextDocumentItemImpl textDocument) {
    this.textDocument = textDocument;
  }
  
  /**
   * Legacy property to support protocol version 1.0 requests.
   */
  @Nullable
  private String text;
  
  @Pure
  @Override
  @Deprecated
  @Nullable
  public String getText() {
    return this.text;
  }
  
  @Deprecated
  public void setText(@Nullable final String text) {
    this.text = text;
  }
  
  public DidOpenTextDocumentParamsImpl() {
    
  }
  
  public DidOpenTextDocumentParamsImpl(final DidOpenTextDocumentParamsImpl other) {
    super(other);
    this.textDocument = other.textDocument;
    this.text = other.text;
  }
  
  @Override
  @Pure
  public String toString() {
    ToStringBuilder b = new ToStringBuilder(this);
    b.add("textDocument", this.textDocument);
    b.add("text", this.text);
    b.add("uri", getUri());
    return b.toString();
  }
  
  @Override
  @Pure
  public boolean equals(final Object obj) {
    if (this == obj)
      return true;
    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    if (!super.equals(obj))
      return false;
    DidOpenTextDocumentParamsImpl other = (DidOpenTextDocumentParamsImpl) obj;
    if (this.textDocument == null) {
      if (other.textDocument != null)
        return false;
    } else if (!this.textDocument.equals(other.textDocument))
      return false;
    if (this.text == null) {
      if (other.text != null)
        return false;
    } else if (!this.text.equals(other.text))
      return false;
    return true;
  }
  
  @Override
  @Pure
  public int hashCode() {
    final int prime = 31;
    int result = super.hashCode();
    result = prime * result + ((this.textDocument== null) ? 0 : this.textDocument.hashCode());
    result = prime * result + ((this.text== null) ? 0 : this.text.hashCode());
    return result;
  }
}
