/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.impl;

import io.typefox.lsapi.ReferenceParams;
import io.typefox.lsapi.impl.ReferenceContextImpl;
import io.typefox.lsapi.impl.TextDocumentPositionParamsImpl;
import org.eclipse.xtext.xbase.lib.Pure;
import org.eclipse.xtext.xbase.lib.util.ToStringBuilder;

/**
 * The references request is sent from the client to the server to resolve project-wide references for the symbol
 * denoted by the given text document position.
 */
@SuppressWarnings("all")
public class ReferenceParamsImpl extends TextDocumentPositionParamsImpl implements ReferenceParams {
  private ReferenceContextImpl context;
  
  @Pure
  @Override
  public ReferenceContextImpl getContext() {
    return this.context;
  }
  
  public void setContext(final ReferenceContextImpl context) {
    this.context = context;
  }
  
  public ReferenceParamsImpl() {
    
  }
  
  public ReferenceParamsImpl(final ReferenceParamsImpl other) {
    super(other);
    this.context = other.context;
  }
  
  @Override
  @Pure
  public String toString() {
    ToStringBuilder b = new ToStringBuilder(this);
    b.add("context", this.context);
    b.add("textDocument", getTextDocument());
    b.add("uri", getUri());
    b.add("position", getPosition());
    return b.toString();
  }
  
  @Override
  @Pure
  public boolean equals(final Object obj) {
    if (this == obj)
      return true;
    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    if (!super.equals(obj))
      return false;
    ReferenceParamsImpl other = (ReferenceParamsImpl) obj;
    if (this.context == null) {
      if (other.context != null)
        return false;
    } else if (!this.context.equals(other.context))
      return false;
    return true;
  }
  
  @Override
  @Pure
  public int hashCode() {
    final int prime = 31;
    int result = super.hashCode();
    result = prime * result + ((this.context== null) ? 0 : this.context.hashCode());
    return result;
  }
}
