/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.impl;

import io.typefox.lsapi.ResponseError;
import io.typefox.lsapi.ResponseErrorCode;
import javax.annotation.Nullable;
import org.eclipse.xtext.xbase.lib.Pure;
import org.eclipse.xtext.xbase.lib.util.ToStringBuilder;

@SuppressWarnings("all")
public class ResponseErrorImpl implements ResponseError {
  /**
   * A number indicating the error type that occured.
   */
  private ResponseErrorCode code;
  
  @Pure
  @Override
  public ResponseErrorCode getCode() {
    return this.code;
  }
  
  public void setCode(final ResponseErrorCode code) {
    this.code = code;
  }
  
  /**
   * A string providing a short decription of the error.
   */
  private String message;
  
  @Pure
  @Override
  public String getMessage() {
    return this.message;
  }
  
  public void setMessage(final String message) {
    this.message = message;
  }
  
  /**
   * A Primitive or Structured value that contains additional information about the error. Can be omitted.
   */
  @Nullable
  private Object data;
  
  @Pure
  @Override
  @Nullable
  public Object getData() {
    return this.data;
  }
  
  public void setData(@Nullable final Object data) {
    this.data = data;
  }
  
  public ResponseErrorImpl() {
    
  }
  
  public ResponseErrorImpl(final ResponseErrorImpl other) {
    this.code = other.code;
    this.message = other.message;
    this.data = other.data;
  }
  
  public ResponseErrorImpl(final ResponseErrorCode code, final String message, final Object data) {
    this.code = code;
    this.message = message;
    this.data = data;
  }
  
  @Override
  @Pure
  public String toString() {
    ToStringBuilder b = new ToStringBuilder(this);
    b.add("code", this.code);
    b.add("message", this.message);
    b.add("data", this.data);
    return b.toString();
  }
  
  @Override
  @Pure
  public boolean equals(final Object obj) {
    if (this == obj)
      return true;
    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    ResponseErrorImpl other = (ResponseErrorImpl) obj;
    if (this.code == null) {
      if (other.code != null)
        return false;
    } else if (!this.code.equals(other.code))
      return false;
    if (this.message == null) {
      if (other.message != null)
        return false;
    } else if (!this.message.equals(other.message))
      return false;
    if (this.data == null) {
      if (other.data != null)
        return false;
    } else if (!this.data.equals(other.data))
      return false;
    return true;
  }
  
  @Override
  @Pure
  public int hashCode() {
    final int prime = 31;
    int result = 1;
    result = prime * result + ((this.code== null) ? 0 : this.code.hashCode());
    result = prime * result + ((this.message== null) ? 0 : this.message.hashCode());
    result = prime * result + ((this.data== null) ? 0 : this.data.hashCode());
    return result;
  }
}
