/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.impl;

import io.typefox.lsapi.ShowMessageRequestParams;
import io.typefox.lsapi.impl.MessageActionItemImpl;
import io.typefox.lsapi.impl.MessageParamsImpl;
import java.util.List;
import javax.annotation.Nullable;
import org.eclipse.xtext.xbase.lib.Pure;
import org.eclipse.xtext.xbase.lib.util.ToStringBuilder;

/**
 * The show message request is sent from a server to a client to ask the client to display a particular message in the
 * user interface. In addition to the show message notification the request allows to pass actions and to wait for an
 * answer from the client.
 */
@SuppressWarnings("all")
public class ShowMessageRequestParamsImpl extends MessageParamsImpl implements ShowMessageRequestParams {
  /**
   * The message action items to present.
   */
  @Nullable
  private List<MessageActionItemImpl> actions;
  
  @Pure
  @Override
  @Nullable
  public List<MessageActionItemImpl> getActions() {
    return this.actions;
  }
  
  public void setActions(@Nullable final List<MessageActionItemImpl> actions) {
    this.actions = actions;
  }
  
  public ShowMessageRequestParamsImpl() {
    
  }
  
  public ShowMessageRequestParamsImpl(final ShowMessageRequestParamsImpl other) {
    super(other);
    this.actions = other.actions;
  }
  
  @Override
  @Pure
  public String toString() {
    ToStringBuilder b = new ToStringBuilder(this);
    b.add("actions", this.actions);
    b.add("type", getType());
    b.add("message", getMessage());
    return b.toString();
  }
  
  @Override
  @Pure
  public boolean equals(final Object obj) {
    if (this == obj)
      return true;
    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    if (!super.equals(obj))
      return false;
    ShowMessageRequestParamsImpl other = (ShowMessageRequestParamsImpl) obj;
    if (this.actions == null) {
      if (other.actions != null)
        return false;
    } else if (!this.actions.equals(other.actions))
      return false;
    return true;
  }
  
  @Override
  @Pure
  public int hashCode() {
    final int prime = 31;
    int result = super.hashCode();
    result = prime * result + ((this.actions== null) ? 0 : this.actions.hashCode());
    return result;
  }
}
