/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.impl;

import io.typefox.lsapi.SymbolInformation;
import io.typefox.lsapi.SymbolKind;
import io.typefox.lsapi.impl.LocationImpl;
import javax.annotation.Nullable;
import org.eclipse.xtext.xbase.lib.Pure;
import org.eclipse.xtext.xbase.lib.util.ToStringBuilder;

/**
 * Represents information about programming constructs like variables, classes, interfaces etc.
 */
@SuppressWarnings("all")
public class SymbolInformationImpl implements SymbolInformation {
  /**
   * The name of this symbol.
   */
  private String name;
  
  @Pure
  @Override
  public String getName() {
    return this.name;
  }
  
  public void setName(final String name) {
    this.name = name;
  }
  
  /**
   * The kind of this symbol.
   */
  private SymbolKind kind;
  
  @Pure
  @Override
  public SymbolKind getKind() {
    return this.kind;
  }
  
  public void setKind(final SymbolKind kind) {
    this.kind = kind;
  }
  
  /**
   * The location of this symbol.
   */
  private LocationImpl location;
  
  @Pure
  @Override
  public LocationImpl getLocation() {
    return this.location;
  }
  
  public void setLocation(final LocationImpl location) {
    this.location = location;
  }
  
  /**
   * The name of the symbol containing this symbol.
   */
  @Nullable
  private String containerName;
  
  @Pure
  @Override
  @Nullable
  public String getContainerName() {
    return this.containerName;
  }
  
  public void setContainerName(@Nullable final String containerName) {
    this.containerName = containerName;
  }
  
  public SymbolInformationImpl() {
    
  }
  
  public SymbolInformationImpl(final SymbolInformationImpl other) {
    this.name = other.name;
    this.kind = other.kind;
    this.location = other.location;
    this.containerName = other.containerName;
  }
  
  @Override
  @Pure
  public String toString() {
    ToStringBuilder b = new ToStringBuilder(this);
    b.add("name", this.name);
    b.add("kind", this.kind);
    b.add("location", this.location);
    b.add("containerName", this.containerName);
    return b.toString();
  }
  
  @Override
  @Pure
  public boolean equals(final Object obj) {
    if (this == obj)
      return true;
    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    SymbolInformationImpl other = (SymbolInformationImpl) obj;
    if (this.name == null) {
      if (other.name != null)
        return false;
    } else if (!this.name.equals(other.name))
      return false;
    if (this.kind == null) {
      if (other.kind != null)
        return false;
    } else if (!this.kind.equals(other.kind))
      return false;
    if (this.location == null) {
      if (other.location != null)
        return false;
    } else if (!this.location.equals(other.location))
      return false;
    if (this.containerName == null) {
      if (other.containerName != null)
        return false;
    } else if (!this.containerName.equals(other.containerName))
      return false;
    return true;
  }
  
  @Override
  @Pure
  public int hashCode() {
    final int prime = 31;
    int result = 1;
    result = prime * result + ((this.name== null) ? 0 : this.name.hashCode());
    result = prime * result + ((this.kind== null) ? 0 : this.kind.hashCode());
    result = prime * result + ((this.location== null) ? 0 : this.location.hashCode());
    result = prime * result + ((this.containerName== null) ? 0 : this.containerName.hashCode());
    return result;
  }
}
