/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.impl;

import io.typefox.lsapi.TextDocumentPositionParams;
import io.typefox.lsapi.impl.PositionImpl;
import io.typefox.lsapi.impl.TextDocumentIdentifierImpl;
import javax.annotation.Nullable;
import org.eclipse.xtext.xbase.lib.Pure;
import org.eclipse.xtext.xbase.lib.util.ToStringBuilder;

/**
 * A parameter literal used in requests to pass a text document and a position inside that document.
 */
@SuppressWarnings("all")
public class TextDocumentPositionParamsImpl implements TextDocumentPositionParams {
  /**
   * The text document.
   */
  private TextDocumentIdentifierImpl textDocument;
  
  @Pure
  @Override
  public TextDocumentIdentifierImpl getTextDocument() {
    return this.textDocument;
  }
  
  public void setTextDocument(final TextDocumentIdentifierImpl textDocument) {
    this.textDocument = textDocument;
  }
  
  /**
   * Legacy property to support protocol version 1.0 requests.
   */
  @Nullable
  private String uri;
  
  @Pure
  @Override
  @Deprecated
  @Nullable
  public String getUri() {
    return this.uri;
  }
  
  @Deprecated
  public void setUri(@Nullable final String uri) {
    this.uri = uri;
  }
  
  /**
   * The position inside the text document.
   */
  private PositionImpl position;
  
  @Pure
  @Override
  public PositionImpl getPosition() {
    return this.position;
  }
  
  public void setPosition(final PositionImpl position) {
    this.position = position;
  }
  
  public TextDocumentPositionParamsImpl() {
    
  }
  
  public TextDocumentPositionParamsImpl(final TextDocumentPositionParamsImpl other) {
    this.textDocument = other.textDocument;
    this.uri = other.uri;
    this.position = other.position;
  }
  
  public TextDocumentPositionParamsImpl(final TextDocumentIdentifierImpl textDocument, final String uri, final PositionImpl position) {
    this.textDocument = textDocument;
    this.uri = uri;
    this.position = position;
  }
  
  @Override
  @Pure
  public String toString() {
    ToStringBuilder b = new ToStringBuilder(this);
    b.add("textDocument", this.textDocument);
    b.add("uri", this.uri);
    b.add("position", this.position);
    return b.toString();
  }
  
  @Override
  @Pure
  public boolean equals(final Object obj) {
    if (this == obj)
      return true;
    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    TextDocumentPositionParamsImpl other = (TextDocumentPositionParamsImpl) obj;
    if (this.textDocument == null) {
      if (other.textDocument != null)
        return false;
    } else if (!this.textDocument.equals(other.textDocument))
      return false;
    if (this.uri == null) {
      if (other.uri != null)
        return false;
    } else if (!this.uri.equals(other.uri))
      return false;
    if (this.position == null) {
      if (other.position != null)
        return false;
    } else if (!this.position.equals(other.position))
      return false;
    return true;
  }
  
  @Override
  @Pure
  public int hashCode() {
    final int prime = 31;
    int result = 1;
    result = prime * result + ((this.textDocument== null) ? 0 : this.textDocument.hashCode());
    result = prime * result + ((this.uri== null) ? 0 : this.uri.hashCode());
    result = prime * result + ((this.position== null) ? 0 : this.position.hashCode());
    return result;
  }
}
