/**
 * Copyright (c) 2016 TypeFox GmbH (http://www.typefox.io) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package io.typefox.lsapi.impl;

import io.typefox.lsapi.WorkspaceEdit;
import io.typefox.lsapi.impl.TextEditImpl;
import java.util.List;
import java.util.Map;
import org.eclipse.xtext.xbase.lib.Pure;
import org.eclipse.xtext.xbase.lib.util.ToStringBuilder;

/**
 * A workspace edit represents changes to many resources managed in the workspace.
 */
@SuppressWarnings("all")
public class WorkspaceEditImpl implements WorkspaceEdit {
  /**
   * Holds changes to existing resources.
   */
  private Map<String, List<TextEditImpl>> changes;
  
  @Pure
  @Override
  public Map<String, List<TextEditImpl>> getChanges() {
    return this.changes;
  }
  
  public void setChanges(final Map<String, List<TextEditImpl>> changes) {
    this.changes = changes;
  }
  
  public WorkspaceEditImpl() {
    
  }
  
  public WorkspaceEditImpl(final WorkspaceEditImpl other) {
    this.changes = other.changes;
  }
  
  public WorkspaceEditImpl(final Map<String, List<TextEditImpl>> changes) {
    this.changes = changes;
  }
  
  @Override
  @Pure
  public String toString() {
    ToStringBuilder b = new ToStringBuilder(this);
    b.add("changes", this.changes);
    return b.toString();
  }
  
  @Override
  @Pure
  public boolean equals(final Object obj) {
    if (this == obj)
      return true;
    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    WorkspaceEditImpl other = (WorkspaceEditImpl) obj;
    if (this.changes == null) {
      if (other.changes != null)
        return false;
    } else if (!this.changes.equals(other.changes))
      return false;
    return true;
  }
  
  @Override
  @Pure
  public int hashCode() {
    final int prime = 31;
    int result = 1;
    result = prime * result + ((this.changes== null) ? 0 : this.changes.hashCode());
    return result;
  }
}
